PRO compute_photo_z_pan_tarrio, dir_name_cat_train, dir_name_cat_test, dir_name_cat_out, feat_type=feat_type, train_type=train_type, std_type=std_type, use_5feat=use_5feat, use_4feat=use_4feat, Nneigh=Nneigh

  ; ***********************************************************************************************************************
  ; This procedure calculates the photometric redshift of a set of galaxies using
  ; the training set and the photometric redshift estimation method described in
  ; Tarrío and Zarattini 2020.

  ; *****************
  ; INPUT PARAMETERS:
  ; *****************
  ; - dir_name_cat_train: Full path and name of the training set (FITS file downloaded with this code)
  ; - dir_name_cat_test:  Full path and name of the set of galaxies for which we will calculate z_phot
  ;                       This set should be in FITS format. 
  ;                       It should contain a catalogue with any number of galaxies. 
  ;                       For each galaxy, it should provide the 5 features to be used for the computation 
  ;                       of the photometric redshift. Two options are admitted:
  ;                       - For aperture colours: R_KRON,GR_APER,RI_APER,IZ_APER,ZY_APER 
  ;                       - For Kron colours:     R_KRON,GR_KRON,RI_KRON,IZ_KRON,ZY_KRON   
  ;                       If a feature is not available, it should be set to -999
  ;                       The user may want to use prepare_test_galaxies.pro to create this file
  ; - dir_name_cat_out:   Full path and name of the output catalogue (where results will be saved)
  ;
  ; *****************
  ; KEYWORDS:
  ; *****************
  ; - feat_type:  Set of features to be used to compute the photometric redshift. Two options are available:
  ;               - 'aper': The 5 features to be used are the g-r, r-i, i-z, and z-y aperture colours and the r-band Kron magnitude 
  ;               - 'kron': The 5 features to be used are the g-r, r-i, i-z, and z-y Kron colours and the r-band Kron magnitude
  ;               The default value is 'aper'.
  ;               See Tarrío and Zarattini 2020 for details on how these colours are defined.
  ;
  ; - train_type: Defines the subset of the full training set that will be used to compute the photometric redshift when a
  ;               feature is missing from a galaxy. Two options are available:
  ;               - 'T5': Subset of the full training set that has the 5 features available
  ;               - 'T4': Subset of the full training set that has the same 4 features as the galaxy
  ;               The default value is 'T5'.
  ;               See Tarrío and Zarattini 2020 for details more details on these definitions.
  ;
  ; - std_type:   Defines how the standardization of the features is done. The standardization of the features
  ;               is done by subtracting the mean and dividing by the standard deviation of the features in a
  ;               given subset of the training set. This keyword defines the subset to be used for calculating
  ;               the mean and standard deviation to be applied in the standardization. Two options are available:
  ;               - 'T5': Subset of the full training set that has the 5 features available
  ;               - 'T1': For each feature, we take the subset of the full training set where that feature is available
  ;               The default value is 'T5'.
  ;               The standardization applies to training set galaxies as well as to the test galaxies.
  ;
  ; - use_5feat:  To compute (1) or not (0) the photometric redshift of galaxies that have the 5 features available.
  ;               The default value is 1
  ;
  ; - use_4feat:  To compute (1) or not (0) the photometric redshift of galaxies that have only 4 features available.
  ;               The default value is 1
  ;               
  ; - Nneigh:     Number of neighbours that will be used in the local linear regression (k in Tarrío and Zarattini 2020)
  ;               The default value is 100
  ;
  ; *****************
  ; OUTPUT:
  ; *****************
  ; The output catalogue contains the same number of galaxies as the input catalogue, in the same order.
  ; For each galaxy, the following fields are provided:
  ; - z_phot: photometric redshift
  ; - z_phot_err: error in the photometric redshift
  ; - n_feat: number of features that were used to calculate the photometric redshift
  ; - flag_extrapolate: flag to indicate if the photometric redshift is calculated via an interpolation (0)
  ;                     or an extrapolation (1) of the training features. Extrapolation occurs when the
  ;                     test galaxy lies outside the bounding box of its k nearest neighbours (in the feature space).
  ;                     See Tarrío and Zarattini 2020 for details more details on this flag.
  ;
  ; *****************
  ; RESTRICTIONS:
  ; *****************
  ; Not fully tested yet. Report any bugs to Paula Tarrío
  ; p.tarrio@oan.es
  ;
  ; *****************
  ; REVISION HISTORY
  ; *****************
  ; Written, Paula Tarrío, May 19 2020
  ;
  ; ***********************************************************************************************************************




  ; ----------------------------------------------------------
  ; Checking keywords
  ; ----------------------------------------------------------
  IF NOT KEYWORD_SET(feat_type) THEN feat_type = 'aper'
  if (feat_type ne 'aper' and feat_type ne 'kron') then begin
    print, 'Error: The selected feat_type is not correct: It should be kron or aper (default).'
    goto, badinput
  endif
  
  IF NOT KEYWORD_SET(train_type) THEN train_type = 'T5'
  if (train_type ne 'T4' and train_type ne 'T5') then begin
    print, 'Error: The selected train_type is not correct: It should be T4 or T5 (default).'
    goto, badinput
  endif

  IF NOT KEYWORD_SET(std_type) THEN std_type = 'T5'
  if (std_type ne 'T1' and std_type ne 'T5') then begin
    print, 'Error: The selected std_type is not correct: It should be T1 or T5 (default).'
    goto, badinput
  endif

  IF NOT KEYWORD_SET(use_5feat) THEN use_5feat = 1
  if (use_5feat ne 1 and use_5feat ne 0) then begin
    print, 'Error: The selected use_5feat is not correct: It should be 0 or 1 (default).'
    goto, badinput
  endif
  
  IF NOT KEYWORD_SET(use_4feat) THEN use_4feat = 1
  if (use_4feat ne 1 and use_4feat ne 0) then begin
    print, 'Error: The selected use_4feat is not correct: It should be 0 or 1 (default).'
    goto, badinput
  endif
  
  IF NOT KEYWORD_SET(Nneigh) THEN Nneigh = 100
  if (Nneigh lt 1) then begin
    print, 'Error: The selected Nneigh is not correct: It should be at least 1 (100 recommended)'
    goto, badinput
  endif
  ; ----------------------------------------------------------




  ; Open training set
  cat = mrdfits(dir_name_cat_train, 1)
  Ntot = n_elements(cat)

  ; Select the features of the training set:
  if feat_type eq 'aper' then features_training_notnorm = [[cat.gr_aper],[cat.ri_aper],[cat.iz_aper],[cat.zy_aper],[cat.r_kron]]
  if feat_type eq 'kron' then features_training_notnorm = [[cat.gr_kron],[cat.ri_kron],[cat.iz_kron],[cat.zy_kron],[cat.r_kron]]

  ; Subsets of the training set
  index_T5 = where(features_training_notnorm[*,0] gt -100 and features_training_notnorm[*,1] gt -100 and features_training_notnorm[*,2] gt -100 and features_training_notnorm[*,3] gt -100 and features_training_notnorm[*,4] gt -100)
  index_T4_0123 = where(features_training_notnorm[*,0] gt -100 and features_training_notnorm[*,1] gt -100 and features_training_notnorm[*,2] gt -100 and features_training_notnorm[*,3] gt -100 and features_training_notnorm[*,4] le -100)
  index_T4_0124 = where(features_training_notnorm[*,0] gt -100 and features_training_notnorm[*,1] gt -100 and features_training_notnorm[*,2] gt -100 and features_training_notnorm[*,3] le -100 and features_training_notnorm[*,4] gt -100)
  index_T4_0134 = where(features_training_notnorm[*,0] gt -100 and features_training_notnorm[*,1] gt -100 and features_training_notnorm[*,2] le -100 and features_training_notnorm[*,3] gt -100 and features_training_notnorm[*,4] gt -100)
  index_T4_0234 = where(features_training_notnorm[*,0] gt -100 and features_training_notnorm[*,1] le -100 and features_training_notnorm[*,2] gt -100 and features_training_notnorm[*,3] gt -100 and features_training_notnorm[*,4] gt -100)
  index_T4_1234 = where(features_training_notnorm[*,0] le -100 and features_training_notnorm[*,1] gt -100 and features_training_notnorm[*,2] gt -100 and features_training_notnorm[*,3] gt -100 and features_training_notnorm[*,4] gt -100)

  ; Calculate the mean and standard deviation that will be used to standardize the features
  if std_type eq 'T5' then begin
    features_training_notnorm_formean = features_training_notnorm[index_T5,*]
    mu_features = mean(features_training_notnorm_formean,dim=1)
    sigma_features = stddev(features_training_notnorm_formean,dim=1)
  endif
  if std_type eq 'T1' then begin
    mu_gr = mean(features_training_notnorm[where(features_training_notnorm[*,0] gt -100), 0],dim=1)
    mu_ri = mean(features_training_notnorm[where(features_training_notnorm[*,1] gt -100), 1],dim=1)
    mu_iz = mean(features_training_notnorm[where(features_training_notnorm[*,2] gt -100), 2],dim=1)
    mu_zy = mean(features_training_notnorm[where(features_training_notnorm[*,3] gt -100), 3],dim=1)
    mu_r  = mean(features_training_notnorm[where(features_training_notnorm[*,4] gt -100), 4],dim=1)
    sigma_gr = stddev(features_training_notnorm[where(features_training_notnorm[*,0] gt -100), 0],dim=1)
    sigma_ri = stddev(features_training_notnorm[where(features_training_notnorm[*,1] gt -100), 1],dim=1)
    sigma_iz = stddev(features_training_notnorm[where(features_training_notnorm[*,2] gt -100), 2],dim=1)
    sigma_zy = stddev(features_training_notnorm[where(features_training_notnorm[*,3] gt -100), 3],dim=1)
    sigma_r  = stddev(features_training_notnorm[where(features_training_notnorm[*,4] gt -100), 4],dim=1)
    mu_features = [mu_gr,mu_ri,mu_iz,mu_zy,mu_r]
    sigma_features = [sigma_gr,sigma_ri,sigma_iz,sigma_zy,sigma_r]
  endif

  ; We standardize the features of the training set:
  features_training = (features_training_notnorm - mu_features##(fltarr(Ntot,1)+1))/(sigma_features##(fltarr(Ntot,1)+1))

  
  ; Open test set
  cat_test = mrdfits(dir_name_cat_test, 1)
  Ntest = n_elements(cat_test)

  if Ntest gt 0 then begin

    ; Select features for the test galaxies
    if feat_type eq 'aper' then features_test_notnorm = [[cat_test.gr_aper],[cat_test.ri_aper],[cat_test.iz_aper],[cat_test.zy_aper],[cat_test.r_kron]]
    if feat_type eq 'kron' then features_test_notnorm = [[cat_test.gr_kron],[cat_test.ri_kron],[cat_test.iz_kron],[cat_test.zy_kron],[cat_test.r_kron]]
    
    ; We normalize the features of the test set:
    features_test = (features_test_notnorm - mu_features##(fltarr(Ntest,1)+1))/(sigma_features##(fltarr(Ntest,1)+1))

    ; Subsets of the test set
    good_5feat = where(features_test[*,0] gt -100 and features_test[*,1] gt -100 and features_test[*,2] gt -100 and features_test[*,3] gt -100 and features_test[*,4] gt -100, n_01234)
    good_0123 = where(features_test[*,0] gt -100 and features_test[*,1] gt -100 and features_test[*,2] gt -100 and features_test[*,3] gt -100 and features_test[*,4] le -100, n_0123)
    good_0124 = where(features_test[*,0] gt -100 and features_test[*,1] gt -100 and features_test[*,2] gt -100 and features_test[*,3] le -100 and features_test[*,4] gt -100, n_0124)
    good_0134 = where(features_test[*,0] gt -100 and features_test[*,1] gt -100 and features_test[*,2] le -100 and features_test[*,3] gt -100 and features_test[*,4] gt -100, n_0134)
    good_0234 = where(features_test[*,0] gt -100 and features_test[*,1] le -100 and features_test[*,2] gt -100 and features_test[*,3] gt -100 and features_test[*,4] gt -100, n_0234)
    good_1234 = where(features_test[*,0] le -100 and features_test[*,1] gt -100 and features_test[*,2] gt -100 and features_test[*,3] gt -100 and features_test[*,4] gt -100, n_1234)

    ; Linear regression to calculate z_phot for the galaxies in the test set that have the 5 features available
    if use_5feat eq 1 then begin
      t = SYSTIME(1)
      index_training = index_T5
      if n_01234 gt 0 then z_phot_5feat = linear_regression_zphot_tarrio(features_training[index_training,*], features_test[good_5feat,*], cat[index_training,*].z_spec, Nneigh, z_phot_err_5feat, flag_extrapolate_5feat)
      PRINT, 'Computing time (5 features): '+strtrim((SYSTIME(1)-t),2) + ' sec'
    endif

    ; Linear regression to calculate z_phot for the galaxies in the test set that have only 4 features available
    if use_4feat eq 1 then begin
      t = SYSTIME(1)
      if train_type eq 'T5' then index_training = index_T5

      ftr = features_training[*,[0,1,2,3]]
      fte = features_test[*,[0,1,2,3]]
      if train_type eq 'T4' then index_training = index_T4_0123
      if n_0123 gt 0 then z_phot_0123 = linear_regression_zphot_tarrio(ftr[index_training,*], fte[good_0123,*], cat[index_training,*].z_spec, Nneigh, z_phot_err_0123, flag_extrapolate_0123)

      ftr = features_training[*,[0,1,2,4]]
      fte = features_test[*,[0,1,2,4]]
      if train_type eq 'T4' then index_training = index_T4_0124
      if n_0124 gt 0 then z_phot_0124 = linear_regression_zphot_tarrio(ftr[index_training,*], fte[good_0124,*], cat[index_training,*].z_spec, Nneigh, z_phot_err_0124, flag_extrapolate_0124)

      ftr = features_training[*,[0,1,3,4]]
      fte = features_test[*,[0,1,3,4]]
      if train_type eq 'T4' then index_training = index_T4_0134
      if n_0134 gt 0 then z_phot_0134 = linear_regression_zphot_tarrio(ftr[index_training,*], fte[good_0134,*], cat[index_training,*].z_spec, Nneigh, z_phot_err_0134, flag_extrapolate_0134)

      ftr = features_training[*,[0,2,3,4]]
      fte = features_test[*,[0,2,3,4]]
      if train_type eq 'T4' then index_training = index_T4_0234
      if n_0234 gt 0 then z_phot_0234 = linear_regression_zphot_tarrio(ftr[index_training,*], fte[good_0234,*], cat[index_training,*].z_spec, Nneigh, z_phot_err_0234, flag_extrapolate_0234)

      ftr = features_training[*,[1,2,3,4]]
      fte = features_test[*,[1,2,3,4]]
      if train_type eq 'T4' then index_training = index_T4_1234
      if n_1234 gt 0 then z_phot_1234 = linear_regression_zphot_tarrio(ftr[index_training,*], fte[good_1234,*], cat[index_training,*].z_spec, Nneigh, z_phot_err_1234, flag_extrapolate_1234)

      PRINT, 'Computing time (4 features): '+strtrim((SYSTIME(1)-t),2) + ' sec'
    endif


    ; Define output catalogue structure
    new_fields = {z_phot:-1., z_phot_err:-1., n_feat:0, flag_extrapolate:-1}
    cat_out = REPLICATE(new_fields, N_ELEMENTS(cat_test))

    ; Save results in output catalogue structure
    if use_5feat eq 1 then begin
      cat_out[good_5feat].z_phot = z_phot_5feat
      cat_out[good_5feat].z_phot_err = z_phot_err_5feat
      cat_out[good_5feat].n_feat = 5
      cat_out[good_5feat].flag_extrapolate = flag_extrapolate_5feat
    endif
    if use_4feat eq 1 then begin
      if n_0123 gt 0 then begin
        cat_out[good_0123].z_phot = z_phot_0123
        cat_out[good_0123].z_phot_err = z_phot_err_0123
        cat_out[good_0123].flag_extrapolate = flag_extrapolate_0123
        cat_out[good_0123].n_feat = 4
      endif
      if n_0124 gt 0 then begin
        cat_out[good_0124].z_phot = z_phot_0124
        cat_out[good_0124].z_phot_err = z_phot_err_0124
        cat_out[good_0124].flag_extrapolate = flag_extrapolate_0124
        cat_out[good_0124].n_feat = 4
      endif
      if n_0134 gt 0 then begin
        cat_out[good_0134].z_phot = z_phot_0134
        cat_out[good_0134].z_phot_err = z_phot_err_0134
        cat_out[good_0134].flag_extrapolate = flag_extrapolate_0134
        cat_out[good_0134].n_feat = 4
      endif
      if n_0234 gt 0 then begin
        cat_out[good_0234].z_phot = z_phot_0234
        cat_out[good_0234].z_phot_err = z_phot_err_0234
        cat_out[good_0234].flag_extrapolate = flag_extrapolate_0234
        cat_out[good_0234].n_feat = 4
      endif
      if n_1234 gt 0 then begin
        cat_out[good_1234].z_phot = z_phot_1234
        cat_out[good_1234].z_phot_err = z_phot_err_1234
        cat_out[good_1234].flag_extrapolate = flag_extrapolate_1234
        cat_out[good_1234].n_feat = 4
      endif
    endif

    ; Save output catalogue in FITS format
    MWRFITS, cat_out, dir_name_cat_out, /CREATE

  endif

  badinput:

END