PRO prepare_test_galaxies_tarrio, ra, dec, r_kron, g, r, i, z, y, feat_type, dir_name_cat_test, dir_name_ebvmap

  ; ***********************************************************************************************************************
  ; This procedure puts the photometric information downloaded from Pan-STARRS for a given galaxy/galaxies
  ; in the appropriate format to be used as input of compute_photo_z_pan_tarrio.pro.
  ; For this, it takes as input the position (ra, dec) and magnitudes (r_kron, g, r, i, z, y) of the
  ; galaxies, corrects the magnitudes from REDDENING, computes the 5 features required by 
  ; compute_photo_z_pan_tarrio.pro, and saves the resulting information in an appropriate FITS file.
  ; 
  ; IMPORTANT: the magnitudes downloaded from Pan-STARRS should be taken from DR2 and STACK PHOTOMETRY.
  ;
  ; *****************
  ; INPUT PARAMETERS:
  ; *****************
  ; - ra: right ascension of the galaxy (in degrees). Vector with N elements (N=number of galaxies)
  ; - dec: declination of the galaxy (in degrees). Vector with N elements
  ; - r_kron: r-band Kron magnitude downloaded from Pan-STARRS (not dereddened). Vector with N elements
  ; - g: g-band (Kron or aperture) magnitude downloaded from Pan-STARRS (not dereddened). Vector with N elements
  ; - r: r-band (Kron or aperture) magnitude downloaded from Pan-STARRS (not dereddened). Vector with N elements
  ; - i: i-band (Kron or aperture) magnitude downloaded from Pan-STARRS (not dereddened). Vector with N elements
  ; - z: z-band (Kron or aperture) magnitude downloaded from Pan-STARRS (not dereddened). Vector with N elements
  ; - y: y-band (Kron or aperture) magnitude downloaded from Pan-STARRS (not dereddened). Vector with N elements
  ; - feat_type: Indicates the type of magnitudes that are being used. Two options are possible:
  ;               - 'aper': The g,r,i,z,y input magnitudes are aperture magnitudes
  ;               - 'kron': The g,r,i,z,y input magnitudes are Kron magnitudes
  ;               See Tarrío and Zarattini 2020 for details on how these magnitudes are defined.
  ; - dir_name_cat_test:  Full path and name of the test set (fits file that will be saved)
  ; - dir_name_ebvmap: Full name and path of the E(B-V) map from Schlegel et al 1998
  ;
  ; *****************
  ; OUTPUT:
  ; *****************
  ; The output catalogue contains the same number of galaxies as the input parameters, in the same order.
  ; It is saved in FITS format. For each galaxy, it provides the 5 features to be used for the computation
  ; of the photometric redshift. Two options are available:
  ; - For aperture colours: R_KRON,GR_APER,RI_APER,IZ_APER,ZY_APER
  ; - For Kron colours:     R_KRON,GR_KRON,RI_KRON,IZ_KRON,ZY_KRON
  ; If a feature is not available, it is set to -999
  ;
  ; *****************
  ; RESTRICTIONS:
  ; *****************
  ; Not fully tested yet. Report any bugs to Paula Tarrío
  ; p.tarrio@oan.es
  ;
  ; *****************
  ; REVISION HISTORY
  ; *****************
  ; Written, Paula Tarrío, May 20 2020
  ;
  ; ***********************************************************************************************************************



  ; Read the E(B-V) map for Schlegel et al 1998 
  ; (downloaded from https://lambda.gsfc.nasa.gov/product/foreground/fg_sfd_get.cfm)
  read_fits_map, dir_name_ebvmap, ebvmap, hd, xhd, nside=nside

  ; Calculate the Healpix pixels corresponding to the input coordinates
  euler,ra,dec,glon,glat,select=1
  ANG2PIX_nest, nside, !dpi/2-glat*!dpi/180, glon*!dpi/180, listpix

  ; Calculate E(B-V) values for each galaxy
  ebv = ebvmap[listpix,0]

  ; We take A_lambda/E(B-V) for bands g,r,i,z from SDSS EDR paper (Stoughton et al. 2002), table 22
  Ag = ebv*3.793
  Ar = ebv*2.751
  Ai = ebv*2.086
  Az = ebv*1.479
  ; We take A_lambda for band y from the effective lambda (Tonry et al 2012, table 4)
  ; and the unreddening flux function of IDL that uses Fitzpatrick 1999 parametrization:
  FM_UNRED, 9620, 1, ebv, ff
  Ay = 2.5*alog10(ff)

  ; Calculate dereddened magnitudes:
  r_kron_dered = r_kron - Ar
  g_dered = g - Ag
  r_dered = r - Ar
  i_dered = i - Ai
  z_dered = z - Az
  y_dered = y - Ay
  
 
  ; Define fields of the output catalogue (the test set)
  fields = {r_kron:double(-999.), $
            gr_kron:double(-999.), ri_kron:double(-999.), iz_kron:double(-999.), zy_kron:double(-999.), $
            gr_aper:double(-999.), ri_aper:double(-999.), iz_aper:double(-999.), zy_aper:double(-999.)}
  
  ; Create and fill the output catalogue (the test set)
  cat_test = REPLICATE(fields, N_ELEMENTS(R_KRON))
  cat_test.r_kron  = r_kron_dered
  cat_test[where(r_kron le -100, /null)].r_kron = -999

  if feat_type eq 'kron' then begin
    cat_test.gr_kron = g_dered-r_dered
    cat_test.ri_kron = r_dered-i_dered
    cat_test.iz_kron = i_dered-z_dered
    cat_test.zy_kron = z_dered-y_dered
    cat_test[where(g_dered le -100 or r_dered le -100 , /null)].gr_kron = -999
    cat_test[where(r_dered le -100 or i_dered le -100 , /null)].ri_kron = -999
    cat_test[where(i_dered le -100 or z_dered le -100 , /null)].iz_kron = -999
    cat_test[where(z_dered le -100 or y_dered le -100 , /null)].zy_kron = -999
  endif

  if feat_type eq 'aper' then begin
    cat_test.gr_aper = g_dered-r_dered
    cat_test.ri_aper = r_dered-i_dered
    cat_test.iz_aper = i_dered-z_dered
    cat_test.zy_aper = z_dered-y_dered
    cat_test[where(g_dered le -100 or r_dered le -100 , /null)].gr_aper = -999
    cat_test[where(r_dered le -100 or i_dered le -100 , /null)].ri_aper = -999
    cat_test[where(i_dered le -100 or z_dered le -100 , /null)].iz_aper = -999
    cat_test[where(z_dered le -100 or y_dered le -100 , /null)].zy_aper = -999
  endif

  ; Save output catalogue in FITS format
  MWRFITS, cat_test, dir_name_cat_test, /CREATE

badinput:

END